-- nature (Minetest 0.4 mod)
-- Nature is slowly capturing the world!
-- Licenced under the GPLv3

local TREE_GROW_DELAY = 7200 -- 2 hrs
local TREE_GROW_CHANCE = 3 -- * 0.5%
local DENSITY = 3 -- allow 3 trunks in one 'cross'
local MINIMUM_LIGHT = 8 -- light needed for the trees to grow

local CACTUS_GROW_DELAY = 7200 -- seconds
local CACTUS_GROW_CHANCE = 4 -- * 0.5%

local PAPYRUS_GROW_DELAY = 7200 -- seconds
local PAPYRUS_GROW_CHANCE = 15 -- * 0.5%

local ABM_CHANCE = 40

-- Randomization
math.randomseed(os.time())

-- Tree growing
minetest.register_abm({
    nodenames = { "default:leaves" },
    interval = TREE_GROW_DELAY,
    chance = ABM_CHANCE,

    action = function(pos, node, active_object_count, active_object_count_wider)
        -- Individual chance for different nodes
        if(math.random(1, 1000) > TREE_GROW_CHANCE) then
            return
        end

        if(minetest.env:get_node_light(pos, nil) < MINIMUM_LIGHT) then
            return
        end
        
        local trunk_count = 0
        local jungle_trunk_count = 0
        
        -- Check for trunks in area
        for i = -1, 1 do
        for j = -1, 0 do
        for k = -1, 1 do
            local current_node = {
                x = pos.x + i,
                y = pos.y + j,
                z = pos.z + k
            }
            if(math.abs(i) + math.abs(j) + math.abs(k) == 1) then
                if(minetest.env:get_node(current_node).name == "default:tree") then
                trunk_count = trunk_count + 1
                end
                if(minetest.env:get_node(current_node).name ==
                "default:jungletree") then
                    jungle_trunk_count = jungle_trunk_count + 1
                end
            end
        end
        end
        end

        local all_trunks = trunk_count + jungle_trunk_count
        
        -- If there is at least 1 trunk and there are not many of them...
        if (all_trunks > 0) and (all_trunks < DENSITY) then
            minetest.env:remove_node(pos)
            if(math.random(1, all_trunks) <= trunk_count) then
                minetest.env:add_node(pos, {name = "default:tree"})
            else
                minetest.env:add_node(pos, {name = "default:jungletree"})
            end
            print ('[nature] A trunk has grown at (' .. pos.x .. ',' .. pos.y .. ',' .. pos.z .. ')')
            for i = -1, 1 do
            for j = -1, 1 do
            for k = -1, 1 do
                local current_node = {
                    x = pos.x + i,
                    y = pos.y + j,
                    z = pos.z + k
                }
                if(minetest.env:get_node(current_node).name == "air") then
                    minetest.env:add_node(current_node, {name = "default:leaves"})
                end
            end
            end
            end
        end
    end
})


-- Cactus growing
minetest.register_abm({
    nodenames = { "default:cactus" },
    interval = CACTUS_GROW_DELAY,
    chance = ABM_CHANCE,
    
    action = function(pos, node, active_object_count, active_object_count_wider)
        -- Individual chance for different nodes
        if(math.random(1, 1000) > CACTUS_GROW_CHANCE) then
            return
        end
        -- Grow with a chance (choosing which side to grow)
        for i = -1, 1 do
        for j = 0, 1 do
        for k = -1, 1 do
            if(math.abs(i) + math.abs(j) + math.abs(k) == 1) then
                if(math.random(1, 30) == 1) then
                    local new_pos = {
                        x = pos.x + i,
                        y = pos.y + j,
                        z = pos.z + k
                    }
                    if(minetest.env:get_node(new_pos).name == "air") then
                        minetest.env:add_node(new_pos, {name = "default:cactus"})
                        print ('[nature] A cactus has grown at (' .. new_pos.x .. ',' .. new_pos.y .. ',' .. new_pos.z .. ')')
                    end
                end
            end
        end
        end
        end
    end
})

-- Papyrus growing
minetest.register_abm({
    nodenames = { "default:papyrus" },
    interval = PAPYRUS_GROW_DELAY,
    CHANCE = ABM_CHANCE,

    action = function(pos, node, active_object_count, active_object_count_wider)
        -- Individual chance for different nodes
        if(math.random(1, 1000) > PAPYRUS_GROW_CHANCE) then
            return
        end
        
        if(minetest.env:get_node_light(pos, nil) < MINIMUM_LIGHT) then
            return
        end
        
        local under = {
            x = pos.x,
            y = pos.y - 1,
            z = pos.z
        }
        -- Grow sides if connected to land from bottom
        if(minetest.env:get_node(under).name == "default:grass")
        or (minetest.env:get_node(under).name == "default:dirt") then
            for i = -1, 1 do
            for j = -1, 1 do
                if(math.abs(i) + math.abs(j) == 1) then
                    if(math.random(1, 30) == 1) then
                        local new_pos = {
                            x = pos.x + i,
                            y = pos.y,
                            z = pos.z + j
                        }
                        if(minetest.env:get_node(new_pos).name == "air") then
                            minetest.env:add_node(new_pos, {name = "default:papyrus"})
                            print('[nature] A papyrus has grown at (' .. new_pos.x .. ',' .. new_pos.y .. ',' .. new_pos.z .. ')')
                        end
                    end
                end
            end
            end
        end
        -- Grow up
        local above = {
            x = pos.x,
            y = pos.y + 1,
            z = pos.z
        }
        if(minetest.env:get_node(above).name == "air") then
            minetest.env:add_node(above, {name = "default:papyrus"})
            print('[nature] A papyrus has grown at (' .. above.x .. ',' .. above.y .. ',' .. above.z .. ')')
        end
    end
})
